@@ -26,7 +26,7 @@
 #endif
 
 #define EXEC_ON_CLICK 1
-#define ASMON_VERSION "0.60"
+#define ASMON_VERSION "0.70rc2"
 #define CHAR_WIDTH 5
 #define CHAR_HEIGHT 7
 
@@ -47,9 +47,12 @@
 #define B_RED   (1)
 #define B_GREEN (2)
 
+
 /* Evil globals I haven't removed yet */
 long last_pageins=0, last_pageouts=0;
 long last_swapins=0, last_swapouts=0;
+//double old;
+static int has_kern26 = 0;
 
 #ifdef EXEC_ON_CLICK
 char Command[256]="";
@@ -84,7 +87,9 @@ int main(int argc, char *argv[])
 	int allmem=1;
 	int Xpid=1;
 	char *ProgName;
-
+        struct utsname name;
+        int kernMajor, kernMinor, kernRev;
+       
 	ProgName = argv[0];
 
 	if (strlen(ProgName) >= 5)
@@ -122,9 +127,6 @@ exit(0);
 #endif
 				Xpid=0;
 				break;
-			case 'm' :
-				allmem=0;
-				break;
 			default:
 				usage();
 				exit(0);
@@ -151,6 +153,23 @@ exit(0);
 #endif
 	/* Open 64x64 window */
 	openXwindow(argc, argv, asmon_master_xpm, asmon_mask_bits, asmon_mask_width, asmon_mask_height);
+	
+    	if( uname( &name ) != -1 )
+    	{
+	    if( strcmp( name.sysname, "Linux" ) == 0 )
+            {
+                sscanf(name.release, "%d.%d.%d", &kernMajor, &kernMinor, 
+				&kernRev);
+		if(( kernMajor == 2 ) && ( kernMinor == 6 ))
+		    has_kern26 = 1;
+	    }
+    	}
+    	else
+    	{
+            fprintf(stderr, "Can't find system name\n");
+            exit(1);
+    	}
+
 	asmon_routine(Xpid, allmem);
 	return(0);
 }
@@ -158,7 +177,7 @@ exit(0);
 /**************************************************************************/
 
 void usage(void) {
-    fprintf(stderr, "\nasmon %s - by Brad Hall (brad@rio.dhs.org)\n\t\toriginally based on Timecop's wmcpu\n\n", ASMON_VERSION);
+    fprintf(stderr, "\nasmon %s - by Brad Hall (bkh@rio.vg)\n\t\toriginally based on Timecop's wmcpu\n\n", ASMON_VERSION);
     fprintf(stderr, "The top bar: left is the CPU usage, right is the load average\n");
     fprintf(stderr, "The middle bar: left memory usage devided by ticks into shared, buffers, and\n\t\t cached, respectively, and the number of megs used\n");
     fprintf(stderr, "The lower bar: the left swap usage and the number of megs swappedd avg\n");
@@ -168,7 +187,6 @@ void usage(void) {
     fprintf(stderr, "\t-v\tprint the version number\n");
 #ifndef __solaris__
     fprintf(stderr, "\t-u\tforce asmon to show uptime, rather than X mem use\n");
-    fprintf(stderr, "\t-m\tmake asmon show only shared mem, like most other mem meters\n");
 #endif
 #ifdef EXEC_ON_CLICK
     fprintf(stderr, "\t-e cmd\texecute 'cmd' on mouse click\n");
@@ -324,37 +342,84 @@ void DrawCPU
 void DrawCPU(void)
 {
 	FILE *fp;
-	static float cpustat[4];   /* remember the statistics read last time */
-	float fields[4], info[4], cputotal=0.0;
-	long pageins=0, pageouts=0, swapins=0, swapouts=0;
+	static double cpustat[7];  /* remember the statistics read last time */
+	//double fields[7], info[7], cputotal=0.0,idlee=0.0;
+	double fields[7], info[7], cputotal=0.0;
+	long pageins=0, pageouts=0, swapins=0, swapouts=0 ;
 	char buf[128];
 	int i;
-
+	
 	if( (fp = fopen("/proc/stat", "r")) != NULL)
 	{
-	    // CPU data
-	    fscanf(fp, "cpu %f %f %f %f", info, info+1, info+2, info+3);
-
-	    // gather data for LED's
-	    while( fgets(buf, 127, fp) )
+	    if( has_kern26 > 0 )
 	    {
-		if (strstr(buf, "page"))
-    		    sscanf(buf, "page %ld %ld", &pageins, &pageouts);
+	        // CPU data
+	        fscanf(fp, "cpu %lf %lf %lf %lf %lf %lf %lf", info, info+1, 
+				info+2, info+3, info+4, info+5, info+6);
 
-		if (strstr(buf, "swap"))
-    		    sscanf(buf, "swap %ld %ld", &swapins, &swapouts);
-	    } 
-	    fclose(fp);
+	        fclose(fp);
 
+	        if( (fp = fopen("/proc/vmstat", "r")) != NULL)
+                {
+	            // gather data for LED's
+	            while( fgets(buf, 127, fp) )
+	            {
+		        if (strstr(buf, "pgpgin"))
+    		            sscanf(buf, "pgpgin %ld", &pageins);
+
+		        if (strstr(buf, "pgpgout"))
+    		            sscanf(buf, "pgpgout %ld", &pageouts);
+
+		        if (strstr(buf, "pswpin"))
+    		            sscanf(buf, "pswpin %ld", &swapins);
+
+		        if (strstr(buf, "pswpout"))
+    		            sscanf(buf, "pswpout %ld", &swapouts);
+	            } 
+	            fclose(fp);
+                }
+            } else {
+	        // CPU data
+	        fscanf(fp, "cpu %lf %lf %lf %lf", info, info+1, info+2, 
+				info+3);
+
+	        // gather data for LED's
+	        while( fgets(buf, 127, fp) )
+	        {
+		    if (strstr(buf, "page"))
+    		        sscanf(buf, "page %ld %ld", &pageins, &pageouts);
+
+		    if (strstr(buf, "swap"))
+    		        sscanf(buf, "swap %ld %ld", &swapins, &swapouts);
+	        } 
+	        fclose(fp);
+            }
+	    
 	    // Calculate CPU stuff
-	    for(i = 0; i < 4; i++)
-	    { 
-		fields[i] = info[i] - cpustat[i]; 
-		cputotal += fields[i]; 
-		cpustat[i] = info[i];
+	    if( has_kern26 > 0 )
+	    {
+	        for(i = 0; i < 7; i++)
+	        { 
+		    fields[i] = info[i] - cpustat[i]; 
+		    cputotal += fields[i]; 
+		    cpustat[i] = info[i];
+	        }
+	    } else {
+	        for(i = 0; i < 4; i++)
+	        { 
+		    fields[i] = info[i] - cpustat[i]; 
+		    cputotal += fields[i]; 
+		    cpustat[i] = info[i];
+	        }
 	    }
-
+	    //idlee=info[3]-old;
+            
+            //old=info[3]; 
+	    
 	    // CPU Bar
+	    
+	    //cputotal = 100 * l1 ;
+	    //cputotal=(100-(idlee*100/16))*26/100;
 	    if(cputotal > 0)
 	    {
 		cputotal = (cputotal-(fields[3]+fields[4]))*1.55;
@@ -408,7 +473,7 @@ void DrawCPU(void)
 void DrawLoad(void)
 {
     int tempy, tempa, i;
-    static float oldv = 0.0;
+    static float oldv = -1.0;
     float ftmp;
 		
 #ifdef __solaris__
@@ -425,7 +490,7 @@ void DrawLoad(void)
 	{
 	    oldv = ftmp;
 	    i=ftmp*26;
-	    tempa=ftmp*100;
+	    tempa=(ftmp+0.005)*100;
 	    tempy=tempa%10;
 	    copyXPMArea(3+(tempy*6),66,6,9,50,5);
 	    tempy=tempa/10;
@@ -549,80 +614,95 @@ float DrawMemSwap(float total, int allmem)
 	    swapused, swapfreeM;
 	unsigned long MEMshar,MEMbuff,MEMcach, MEMswap;
 	int tempy, tempa;
-			
-	fgets(junk, 80, fp);
-	fscanf(fp, "Mem: %f %f %f %f %f %f\nSwap: %f %f %f",  &total, &used, &freeM, &shared, &buffers, &cached, &swaptotal, &swapused, &swapfreeM);
+
+	if( has_kern26 > 0 )
+	{
+	    float scratch;
+	
+	    while(!feof(fp))
+	    {
+		fgets(junk, 120, fp);
+                if (strstr(junk, "MemTotal"))
+                {
+                    sscanf(junk, "MemTotal: %f kB", &scratch);
+		    total = scratch * 1024;
+                }
+                if (strstr(junk, "MemFree"))
+                {
+                    sscanf(junk, "MemFree: %f kB", &scratch);
+		    freeM = scratch * 1024;
+		    used = total - freeM;
+                }
+                if (strstr(junk, "Buffers"))
+                {
+                    sscanf(junk, "Buffers: %f kB", &scratch);
+		    buffers = scratch * 1024;
+                }
+                if (strstr(junk, "Cached"))
+                {
+                    sscanf(junk, "Cached: %f kB", &scratch);
+		    cached = scratch * 1024;
+                }
+                if (strstr(junk, "SwapTotal"))
+                {
+                    sscanf(junk, "SwapTotal: %f kB", &scratch);
+		    swaptotal = scratch * 1024;
+                }
+                if (strstr(junk, "SwapFree"))
+                {
+                    sscanf(junk, "SwapFree: %f kB", &scratch);
+		    swapfreeM = scratch * 1024;
+		    swapused = swaptotal - swapfreeM;
+                }
+            }
+	} else {
+	    fgets(junk, 80, fp);
+	    fscanf(fp, "Mem: %f %f %f %f %f %f\nSwap: %f %f %f",  &total, 
+			    &used, &freeM, &shared, &buffers, &cached,
+			    &swaptotal, &swapused, &swapfreeM);
+	}
 	fclose(fp);
 
-	if ( allmem == 1) {
 	/* All mem areas */
-		if(stotal != total || sshared != shared || sbuffers != buffers || scached != cached)
+	if(stotal != total || sshared != shared || sbuffers != buffers || scached != cached)
+	{
+		stotal = total; sshared = shared; sbuffers = buffers; scached = cached;
+		if ( (total/101048576) >= 1) 
 		{
-			stotal = total; sshared = shared; sbuffers = buffers; scached = cached;
-			if ( (total/101048576) >= 1) 
-			{
-			    MEMshar=(shared/total)*27;
-			    MEMbuff=(buffers/total)*27;
-			    MEMcach=(cached/total)*27;
-			} else {
-			    MEMshar=(shared/total)*33;
-			    MEMbuff=(buffers/total)*33;
-			    MEMcach=(cached/total)*33;
-			}
-                        // refresh
-                        copyXPMArea(4, 115, 55, 11, 4, 18);
-			// Bar
-			if ( (total/101048576) >= 1) 
-			{
-				copyXPMArea(3,75,((used/total)*28),9,5,19);
-			} else {
-				copyXPMArea(3,75,((used/total)*34),9,5,19);
-			}
-			// Separators
-			copyXPMArea(15,105,1,9,5+MEMshar,19);
-			copyXPMArea(15,105,1,9,7+MEMshar+MEMbuff,19);
-			copyXPMArea(15,105,(36-(used/total)*34),9,(5+(used/total)*34),19);
-			// Numbers			
-			tempa=used/1048576;
-			tempy=tempa%10;
-    			copyXPMArea(3+(tempy*6),66,6,9,50,19);
-			tempy=(tempa/10)%10;
-			copyXPMArea(3+(tempy*6),66,6,9,44,19);
-			tempy=(tempa/100)%10;
-			if ( (total/101048576) >= 1) {
-			    copyXPMArea(3+(tempy*6),66,6,9,38,19);
-			    copyXPMArea(16,46,2,14,35,16);  
-			} else {
-			    copyXPMArea(16,46,2,14,41,16); 
-			} 
+		    MEMshar=((used-buffers-cached)/total)*27;
+		    MEMbuff=(buffers/total)*27;
+		    MEMcach=(cached/total)*27;
+		} else {
+		    MEMshar=((used-buffers-cached)/total)*33;
+		    MEMbuff=(buffers/total)*33;
+		    MEMcach=(cached/total)*33;
 		}
-	} else {
-	/* Only Shared Memory */ 
-		if(stotal != total || sshared != shared)
+                // refresh
+                copyXPMArea(4, 115, 55, 11, 4, 18);
+		// Bar
+		if ( (total/101048576) >= 1) 
 		{
-			stotal = total; sshared = shared; sbuffers = buffers; scached = cached;
-			if ( (total/101048576) >= 1) {
-			    MEMshar=(shared/total)*27;
-			} else {
-			    MEMshar=(shared/total)*35;
-			}
-			// Bar
-			copyXPMArea(3,75,MEMshar,9,5,19);
-			copyXPMArea(15,105,(36-(shared/total)*36),9,(5+(shared/total)*36),19);
-			// Numbers			
-			tempa=shared/1048576;
-			tempy=tempa%10;
-    			copyXPMArea(3+(tempy*6),66,6,9,50,19);
-			tempy=(tempa/10)%10;
-			copyXPMArea(3+(tempy*6),66,6,9,44,19);
-			tempy=(tempa/100)%10;
-			if ( (total/101048576) >= 1) {
-			    copyXPMArea(3+(tempy*6),66,6,9,38,19);
-			    copyXPMArea(16,46,2,14,35,16);  
-			} else {
-			copyXPMArea(16,46,2,14,41,16); 
-			} 
+			copyXPMArea(3,75,((used/total)*28),9,5,19);
+		} else {
+			copyXPMArea(3,75,((used/total)*34),9,5,19);
 		}
+		// Separators
+		copyXPMArea(15,105,1,9,5+MEMshar,19);
+		copyXPMArea(15,105,1,9,7+MEMshar+MEMbuff,19);
+		copyXPMArea(15,105,(36-(used/total)*34),9,(5+(used/total)*34),19);
+		// Numbers			
+		tempa=used/1048576;
+		tempy=tempa%10;
+    		copyXPMArea(3+(tempy*6),66,6,9,50,19);
+		tempy=(tempa/10)%10;
+		copyXPMArea(3+(tempy*6),66,6,9,44,19);
+		tempy=(tempa/100)%10;
+		if ( (total/101048576) >= 1) {
+		    copyXPMArea(3+(tempy*6),66,6,9,38,19);
+		    copyXPMArea(16,46,2,14,35,16);  
+		} else {
+		    copyXPMArea(16,46,2,14,41,16); 
+		} 
 	}
 	/* SWAP Meter */
 	if ( swaptotal == 0 ) MEMswap = 0; 
@@ -640,12 +720,12 @@ float DrawMemSwap(float total, int allmem)
 	tempa=swapused/1048576;
 	tempy=tempa%10;
 	copyXPMArea(3+(tempy*6),66,6,9,50,33);
-	tempy=tempa/10;
+	tempy=(tempa/10)%10;
 	copyXPMArea(3+(tempy*6),66,6,9,44,33);
 	tempy=tempa/100;
 	if ( tempy != 0 ) {
 	    copyXPMArea(3+(tempy*6),66,6,9,38,33);
-	    copyXPMArea(16,46,2,14,42,16); 
+	    copyXPMArea(16,46,2,14,35,30); 
 	} else {
 	    copyXPMArea(16,46,2,14,41,30); 
 	}	
@@ -678,7 +758,7 @@ void DrawXmem(int Xpid, float total)
 	}
         if(old_Xsize!=Xsize)
 	{
-    	    int tempy, tempa;
+    	    int tempy, tempa, tempb;
 	    old_Xsize=Xsize;
 	    ratio=Xsize/(total/1024);
 	    if ( Xsize > (total/1024) ) Xsize=total/1024;
@@ -687,10 +767,20 @@ void DrawXmem(int Xpid, float total)
 	    copyXPMArea(3+(tempy*6),66,6,9,50,48);
 	    tempa=Xsize/10;
 	    tempy=tempa%10;
-	    copyXPMArea(3+(tempy*6),66,6,9,44,48); 
-	    copyXPMArea(16,46,2,14,41,46);  
-	    copyXPMArea(3,84,((ratio)*22),11,18,47);
-	    copyXPMArea(15,105,(23-((ratio)*22)),11,(18+(ratio*22)),47);
+	    tempb=Xsize/100;
+	    if ( Xsize > 100 )
+	    {
+	    	copyXPMArea(3,84,((ratio)*17),11,18,47);
+	    	copyXPMArea(15,105,(23-((ratio)*17)),11,(18+(ratio*22)),47);
+	    	copyXPMArea(3+(tempy*6),66,6,9,44,48); 
+		copyXPMArea(3+(tempb*6),66,6,9,38,48);
+	    	copyXPMArea(16,46,2,14,36,46);  
+	    } else {
+	    	copyXPMArea(3,84,((ratio)*22),11,18,47);
+	    	copyXPMArea(15,105,(23-((ratio)*22)),11,(18+(ratio*22)),47);
+	    	copyXPMArea(3+(tempy*6),66,6,9,44,48); 
+	    	copyXPMArea(16,46,2,14,41,46);  
+	    }
 	}
 	fclose(fp);
     }
@@ -736,7 +826,7 @@ void DrawUptime(void)
 	    	copyXPMArea(63,66,3,9,41,48);
     	    	tempy=days%10;
 		copyXPMArea(3+(tempy*6),66,6,9,34,48);
-	    	tempy=days/10;
+	    	tempy=(days/10)%10;
 		copyXPMArea(3+(tempy*6),66,6,9,28,48);
 		tempy=days/100;
 		copyXPMArea(3+(tempy*6),66,6,9,22,48);
